# --------------------------------------------------------------------

server <- function(input, output, session) {

 library(shinyjqui)
 library(shiny)  
 library("plot3D")
 library(ggplot2)
  
  jqui_draggable(ui = "#runtgrafen")
  jqui_draggable(ui = "#runtgrafen2")
  jqui_draggable(ui = "#runtgrafen3")
  jqui_draggable(ui = "#info")
  jqui_draggable(ui = "#kommentarer")
  jqui_draggable(ui = "#ovningar")
  jqui_resizable(ui = ".infoyta")
  jqui_draggable(ui = "#sammanfattning")
  jqui_draggable(ui = "#alldata")
  jqui_resizable(ui = "#alldata")
  
 observe({
   n           <- input$antalvarden; 
   resi        <- input$residual;
   Xkoeff      <- input$Xkoeff; 
   X2koeff     <- input$X2koeff; 
   modell      <- input$modell; 
   

    X <- runif(n, -5, 5)
if(modell == 2){    
    Y <- 52.5 + Xkoeff*X + X2koeff*X*X + rnorm(n, 0, resi)
    model <- lm(Y ~ X + I(X^2))
    mat_coeff <- summary(model)$coefficients
    b0 <- mat_coeff["(Intercept)", 1]
    b1 <- mat_coeff["X", 1]
    b2 <- mat_coeff["I(X^2)", 1]
}    
    
if(modell == 1){    
      Y <- 52.5 + Xkoeff*X + X2koeff*X*X + rnorm(n, 0, resi)   # Medvetet 2:a grad i data
      model <- lm(Y ~ X)                                       # men rätt modell.
      mat_coeff <- summary(model)$coefficients
      b0 <- mat_coeff["(Intercept)", 1]
      b1 <- mat_coeff["X", 1]
}       

      residualer <- resid(model)
output$grafen2 <- renderPlot({
      
      residualdata <- data.frame(residualvarden = residualer)
      binsize <- diff(range(residualdata$residualvarden))/10
      ggplot(residualdata, aes(x=residualvarden)) + geom_histogram(binwidth=binsize, fill="white", colour="blue")   
      
      
#     plot(range(-3:3), range(-3.5:3.5), type = "n", main = "Normalfördelningsdiagram", xlab = "Residualer (Skillnaden mellan modell och data)", ylab ="", yaxt='n')      # skapar en tom graf
#    x <- rnorm(90, 0, 1)
#    points(qqnorm(residualer, plot.it=F, datax=TRUE), col = 3, pch = 16, cex = 1.5)
#    qqline(x, datax=TRUE, col = 3)
      })

output$grafen3 <- renderPlot({ 
  
   plot(range(residualer), range(-3:3), type = "n", main = "Normalfördelningsdiagram", xlab = "Residualer (Skillnaden mellan modell och data)", ylab ="", yaxt='n')      # skapar en tom graf
   points(qqnorm(residualer, plot.it=F, datax=TRUE), col = 3, pch = 16, cex = 1.5)
    qqline(residualer, datax=TRUE, col = 3)
       
})


      output$divtext <- renderPrint({summary(model)}) 

output$grafen <- renderPlot({
    
    XYData <- data.frame(X, Y)
    Xres <- round(X, 2)
    Yres <- round(Y, 2)
    
   output$allmindata <- renderPrint(data.frame(Xres, Yres)) 
    
    ymax = max(Y)
    rubrik <- "Modell 1"
    
    if(modell == 1) {rubrik <- "Modell 1"}
    if(modell == 2) {rubrik <- "Modell 2"}
    
    
    mittDiagramMod2 <- ggplot(XYData, aes(x = X, y = Y, colour = "1")) + annotate("text", x=0, y=ymax, label=rubrik,  fontface="bold", color="blue", size=7) + geom_point() + theme(legend.position="none")

    xmin = min(X)
    xmax = max(X)
    
    Xpred <- seq(xmin, xmax, length.out = 100)
    
    if(modell == 1)    {Ypred <- b0 + b1*Xpred }
    if(modell == 2)    {Ypred <- b0 + b1*Xpred + b2*Xpred*Xpred}
    
    preddata <- data.frame(Xpred, Ypred)
    
    mittDiagramMod2 + geom_line(data=preddata, aes(x=Xpred, y=Ypred, color="2"))
 }) 
})


observeEvent(input$reset, {
  updateSliderInput(inputId = "residual", value = 2.5)
  updateSliderInput(inputId = "Xkoeff", value = -2.1)
  updateSliderInput(inputId = "X2koeff", value = 2.2)
  updateSliderInput(inputId = "antalvarden", value = 30)
  updateSliderInput(inputId = "modell", value = 1)
})

}
